!====================initialization and mapping functions of the x-Q-grids====================
!=============================================================================================
      subroutine INITGRID
      implicit none
c  initialization of the x and Q grid using y=x**a and t=Ln(Ln(Q/b))
      integer :: i
      include "grid.inc"
      include "var.inc"

      if(ang.eq.4) then
      xd=0.9d0*(2.d0*masscutd/sqrts)**2
      qd=0.4d0*masscutd
      qu=2.5d0*masscutu
      else
      xd=0.9d0*(masscutd/sqrts)**2
      qu=1.d4
      if(ang.eq.3) then
      qd=20.d0
      elseif(ang.eq.1) then
      qd=masscutd/dsqrt(chiu+1.d0/chiu+2)
      if(scalescheme.eq.0) then
      qd=0.4d0*qd
      else
      qd=0.4d0*chid**(0.3d0/2.d0)*qd
      endif
      else
      qd=masscutd/dsqrt(dexp(2*chiu)+1.d0/dexp(2*chiu)+2)
      if(scalescheme.eq.0) then
      qd=0.4d0*qd
      else
      qd=0.4d0*dexp(2.d0*chid)**(0.3d0/2.d0)*qd
      endif
      endif
      qd=max(qd, 20.d0)
      endif

      yd=xd**xpo
      yu=xu**xpo
      td=dlog(dlog(qd/Qbase))
      tu=dlog(dlog(qu/Qbase))
      
      do i=1, Nx
      yi(i)=yd+(i-1.d0)*(yu-yd)/(Nx-1.d0)
      xi(i)=yi(i)**(1.d0/xpo)
      enddo
      do i=1, Nq
      ti(i)=td+(i-1.d0)*(tu-td)/(Nq-1.d0)
      qi(i)=Qbase*dexp(dexp(ti(i)))
      enddo

      end subroutine

      subroutine XMAP(XX, LP, PR)
      implicit none
c  return the lower end of x mapping points (4th order) and Projections
      integer :: LP, JLx, JU, JM, Jx, I
      include "grid.inc"
      double precision :: XX, YY, PR(pint+1), iny(pint+1)

      if(XX.lt.-0.0001d0.or.xx.gt.1.0001d0) then
      print *, "warning : XX in Xmap out of range", XX
      stop
      endif

      JLx = 0
      JU = Nx+1
 11   if (JU-JLx .gt. 1) then
      JM = (JU+JLx) / 2
      if (XX .ge. XI(JM)) then
      JLx = JM
      else
      JU = JM
      endif
      goto 11
      endif

      if (JLx .le. 1) then
      Jx = 1
      elseif (JLx .LE. Nx+1-pint) then
      Jx = JLx - 1
      else
      Jx = Nx - pint
      endif

      LP= Jx
      YY=XX**xpo
      do i=1, pint+1
      iny(i)=YI(Jx+i-1)
      enddo
      call POLINT5(iny, YY, PR)

      end subroutine

      subroutine QMAP(QQ, LP, PR)
      implicit none
c  return the lower end of Q mapping points (4th order) and Projections
      integer :: LP, JLx, JU, JM, Jx, I
      include "grid.inc"
      double precision :: QQ, TT, PR(pint+1), iny(pint+1)

      if(QQ.lt.10.d0) then
      print *, "warning : QQ in Qmap out of range", QQ
      QQ=10.d0
      endif

      JLx = 0
      JU = Nq+1
 11   if (JU-JLx .gt. 1) then
      JM = (JU+JLx) / 2
      if (QQ .ge. QI(JM)) then
      JLx = JM
      else
      JU = JM
      endif
      goto 11
      endif

      if (JLx .le. 1) then
      Jx = 1
      elseif (JLx .LE. Nq+1-pint) then
      Jx = JLx - 1
      else
      Jx = Nq - pint
      endif

      LP= Jx
      TT=dlog(dlog(QQ/Qbase))
      do i=1, pint+1
      iny(i)=TI(Jx+i-1)
      enddo
      call POLINT5(iny, TT, PR)

      end subroutine

      subroutine POLINT5(XA, X, PR)
      implicit none
c  interpolation functions at 4th order
      double precision :: XA(5), C(5), B(5), X, PR(5)
      double precision :: D12, D13, D14, D15, D23, D24, D25
      double precision :: D34, D35, D45 
       
      B=1.d0
      B(1)=(X-XA(2))*(X-XA(3))*(X-XA(4))*(X-XA(5))
      B(2)=(X-XA(1))*(X-XA(3))*(X-XA(4))*(X-XA(5))
      B(3)=(X-XA(1))*(X-XA(2))*(X-XA(4))*(X-XA(5))
      B(4)=(X-XA(1))*(X-XA(2))*(X-XA(3))*(X-XA(5))
      B(5)=(X-XA(1))*(X-XA(2))*(X-XA(3))*(X-XA(4))

      D12=XA(1)-XA(2)
      D13=XA(1)-XA(3)
      D14=XA(1)-XA(4)
      D15=XA(1)-XA(5)

      D23=XA(2)-XA(3)
      D24=XA(2)-XA(4)
      D25=XA(2)-XA(5)

      D34=XA(3)-XA(4)
      D35=XA(3)-XA(5)

      D45=XA(4)-XA(5)

c  check x points 

      if((D12*D13*D14*D15*D23*D24*D25*D34*D35*D45).eq.0.d0) then
      print *, "Identical x point in the intepolation grid!"
      print *, XA
      stop
      endif        

      C(1)=1.d0/D12/D13/D14/D15
      C(2)=-1.d0/D12/D23/D24/D25
      C(3)=1.d0/D13/D23/D34/D35
      C(4)=-1.d0/D14/D24/D34/D45
      C(5)=1.d0/D15/D25/D35/D45

      PR=C*B

      end subroutine


      subroutine POLINT6(XA, X, PR)
      implicit none
c  interpolation functions at 5th order
      double precision :: XA(6), C(6), B(6), X, PR(6)
      double precision :: D12, D13, D14, D15, D16, CH
      double precision :: D23, D24, D25, D26
      double precision :: D34, D35, D36, D45, D46, D56 
       
      B=1.d0
      B(1)=(X-XA(2))*(X-XA(3))*(X-XA(4))*(X-XA(5))*(X-XA(6))
      B(2)=(X-XA(1))*(X-XA(3))*(X-XA(4))*(X-XA(5))*(X-XA(6))
      B(3)=(X-XA(1))*(X-XA(2))*(X-XA(4))*(X-XA(5))*(X-XA(6))
      B(4)=(X-XA(1))*(X-XA(2))*(X-XA(3))*(X-XA(5))*(X-XA(6))
      B(5)=(X-XA(1))*(X-XA(2))*(X-XA(3))*(X-XA(4))*(X-XA(6))
      B(6)=(X-XA(1))*(X-XA(2))*(X-XA(3))*(X-XA(4))*(X-XA(5))

      D12=XA(1)-XA(2)
      D13=XA(1)-XA(3)
      D14=XA(1)-XA(4)
      D15=XA(1)-XA(5)
      D16=XA(1)-XA(6)

      D23=XA(2)-XA(3)
      D24=XA(2)-XA(4)
      D25=XA(2)-XA(5)
      D26=XA(2)-XA(6)

      D34=XA(3)-XA(4)
      D35=XA(3)-XA(5)
      D36=XA(3)-XA(6)

      D45=XA(4)-XA(5)
      D46=XA(4)-XA(6)

      D56=XA(5)-XA(6)
c  check x points 

      CH=D12*D13*D14*D15*D23*D24*D25*D34*D35*D45
      CH=CH*D16*D26*D36*D46*D56

      if(CH.eq.0.d0) then
      print *, "Identical x point in the intepolation grid!"
      print *, XA
      stop
      endif        

      C(1)=1.d0/D12/D13/D14/D15/D16
      C(2)=-1.d0/D12/D23/D24/D25/D26
      C(3)=1.d0/D13/D23/D34/D35/D36
      C(4)=-1.d0/D14/D24/D34/D45/D46
      C(5)=1.d0/D15/D25/D35/D45/D56
      C(6)=-1.d0/D16/D26/D36/D46/D56

      PR=C*B

      end subroutine
