!====================subroutine for the phase space generation========
!==============================and integration========================
c 1/2pi*dmkk**2/dx
      subroutine generatemass(x, mdown, mup, mass, jacob) 
c generate resonance mass 
      implicit none
      double precision x, jacob, mass, mdown, mup
      include "var.inc"

      if(mdown.gt.mup) then
             print *, "warning: wrong mass limit!"
             jacob=0.d0
             mass=mup
      else
      mass=mdown+x*(mup-mdown)
      jacob=jacob*(mup-mdown)*2.d0*mass/2.d0/pi 

      endif 

      end subroutine
!=====================================================================
!=====================================================================
      subroutine generatephase1to2(x, y, m1, m2, m3, p, jacob)
c generate a 1->2 phase space
      implicit none
      double precision x, y, jacob, p(0:3,2), pout, m1, m2, m3
      double precision costheta, phi, sintheta, threemomentum
      external threemomentum
      include "var.inc"

      pout=threemomentum(m1, m2, m3)
      phi=2.d0*pi*x
      jacob=2.d0*pi*jacob 
      costheta=-1.d0+2.d0*y
      sintheta=dsqrt(1.d0-costheta**2)
      jacob=2.d0*jacob
c phase space factor
      jacob=pout/16.d0/pi**2/m1*jacob

      p(0,1)=dsqrt(m2**2+pout**2)
      p(1,1)=pout*sintheta*dcos(phi)
      p(2,1)=pout*sintheta*dsin(phi)
      p(3,1)=pout*costheta

      p(0,2)=dsqrt(m3**2+pout**2)
      p(1,2)=-pout*sintheta*dcos(phi)
      p(2,2)=-pout*sintheta*dsin(phi)
      p(3,2)=-pout*costheta


      end subroutine
!=====================================================================
!=====================================================================
      subroutine generatephase2to1(x, mm, frac1, frac2, p, jacob)
c generate a 2->1 phase space
      implicit none
      double precision x, mm, frac1, frac2, p(0:3,2), jacob
      double precision tau
      include "var.inc"

      if((mm.gt.sqrtS)) then
              print *, "wrong generate_phase!"
              tau=0.9d0
              jacob=0.d0
      else
              tau=mm**2/sqrtS**2
      endif

      frac1=tau+(1.d0-tau)*x
      frac2=tau/frac1
      jacob=jacob*(1.d0-tau)*2.d0*pi/sqrtS**2/frac1

      p(0,1)=mm/2.d0
      p(1,1)=0.d0
      p(2,1)=0.d0
      p(3,1)=mm/2.d0

      p(0,2)=mm/2.d0
      p(1,2)=0.d0
      p(2,2)=0.d0
      p(3,2)=-mm/2.d0

      end subroutine
!=====================================================================
!=====================================================================
      double precision function threemomentum(m1, m2, m3)
c generate the three-momentum of the m1->m2,m3 decay 
      implicit none
      double precision m1, m2, m3, psq

      if (m1.lt.(m2+m3)) then
              print *, "warning: wrong 3 momentum!"
              threemomentum=0.d0
              return
      endif

      psq=abs((m1**2+m2**2-m3**2)**2/4.d0/m1**2-m2**2)

      threemomentum=dsqrt(psq)

      end function
!=====================================================================
!=====================================================================
      subroutine pboost(pout,pin,beta,n)
c performing the Lorentz boost
      implicit none
      integer n,i
      double precision pin(0:3,n),pout(0:3,n),beta,gama

      if((beta.gt.1.d0).or.(beta.lt.-1.d0)) then
              print *, "wrong beta!", beta
              beta=0.99999d0*beta/abs(beta)
      endif

      gama=1.d0/sqrt(1-beta*beta)
      
      do i=1,n
      pout(0,i)=gama*(pin(0,i)+beta*pin(3,i))
      pout(1,i)=pin(1,i)
      pout(2,i)=pin(2,i)
      pout(3,i)=gama*(pin(3,i)+beta*pin(0,i))
      enddo

      end subroutine
!=====================================================================
!=====================================================================
      subroutine protation(pout,pin,costheta,n)
c performing a 3-dimensional rotation      
      implicit none
      integer n,i
      double precision pin(0:3,n),pout(0:3,n),costheta,sintheta

      if(abs(costheta).gt.1.d0) then
c              print *, "wrong costheta!"
c              stop
      endif

      sintheta=dsqrt(1.d0-costheta**2)

      do i=1,n
      pout(0,i)=pin(0,i)
      pout(1,i)=pin(1,i)
      pout(2,i)=pin(2,i)*costheta+pin(3,i)*sintheta
      pout(3,i)=pin(3,i)*costheta-pin(2,i)*sintheta
      enddo

      end subroutine
!=====================================================================
!=====================================================================
      subroutine inverseprotation(pout,pin,costheta,n)
c performing an inverse 3-dimensional rotation      
      implicit none
      integer n,i
      double precision pin(0:3,n),pout(0:3,n),costheta,sintheta

      if(abs(costheta).gt.1.d0) then
c              print *, "wrong costheta!"
c              stop
      endif

      sintheta=-dsqrt(1.d0-costheta**2)

      do i=1,n
      pout(0,i)=pin(0,i)
      pout(1,i)=pin(1,i)
      pout(2,i)=pin(2,i)*costheta+pin(3,i)*sintheta
      pout(3,i)=pin(3,i)*costheta-pin(2,i)*sintheta
      enddo

      end subroutine
!=====================================================================
!=====================================================================
      subroutine ptransform(pout,pin,pmother,n)
c combination of boost and rotation
      implicit none
      integer n,i
      double precision pin(0:3,n),pout(0:3,n),beta,costheta
      double precision paux(0:3,n),pmother(0:3),p

      p=dsqrt(pmother(1)**2+pmother(2)**2+pmother(3)**2)
      beta=p/pmother(0)
      costheta=pmother(3)/p

      call pboost(paux, pin, beta, n)
      call protation(pout, paux, costheta, n)

      end subroutine
!=====================================================================
!=====================================================================
      subroutine inverseptransform(pout,pin,pmother,n)
c combination of boost and inverse rotation
      implicit none
      integer n,i
      double precision pin(0:3,n),pout(0:3,n),beta,costheta
      double precision paux(0:3,n),pmother(0:3),p

      p=dsqrt(pmother(1)**2+pmother(2)**2+pmother(3)**2)
      beta=p/pmother(0)
      costheta=pmother(3)/p

      call inverseprotation(paux, pin, costheta, n)
      call pboost(pout, paux, -beta, n)

      end subroutine
C*********************************************************************
      subroutine cuba(integrand, result, error, nd, nc, key)
c interface to CUBA library
      implicit none
      integer  nd, nc, gridno, neval, fail
      integer  nbatch, statefilelen, key
      double precision   result(nc), userdata
      double precision   error(nc)
      double precision  prob(nc)
      character*100 statefile
      integer  mineval
      integer  maxeval
      integer  flags
      integer  nstart
      integer  nincrease
      integer  iseed
      double precision relac, absac
      common /gvegas/ maxeval, iseed, nstart, nincrease
      external  vegas
      external  integrand

      relac=1.d-100
      absac=1.d-100
      mineval=0
      nstart=100000
      nincrease=50001
      flags=1
      nbatch=1000
      gridno=key
      userdata=1.d0

      call  Vegas(nd, nc, integrand, userdata,
     -    relac, absac, flags, iseed,
     -    mineval, maxeval, nstart, nincrease, 
     -    nbatch, gridno, statefile, neval, fail,
     -    result, error, prob)

      end subroutine cuba
C***********************************************************************
