      subroutine fvixxx(fi,vc,fvi)
c
c This subroutine computes an off-shell fermion wavefunction from a
c flowing-IN external fermion and a vector boson.
c
c input:
c       complex fi(6)          : flow-in  fermion                   |fi>
c       complex vc(6)          : input    vector                      v
c       real    fmass          : mass  of output fermion f'
c
c output:
c       complex fvi(6)         : off-shell fermion             |f',v,fi>
c
      implicit none
      double complex fi(6),vc(6),fvi(6),d
      double precision pf(4),pf2
      
      double precision rZero, rOne
      parameter( rZero = 0.0d0, rOne = 1.0d0 )
      double complex cImag, cZero
      parameter( cImag = ( 0.0d0, 1.0d0 ), cZero = ( 0.0d0, 0.0d0 ) )


      fvi(5) = fi(5)-vc(5)
      fvi(6) = fi(6)-vc(6)

      pf(1) = dble( fvi(5))
      pf(2) = dble( fvi(6))
      pf(3) = dimag(fvi(6))
      pf(4) = dimag(fvi(5))
      pf2 = pf(1)**2-(pf(2)**2+pf(3)**2+pf(4)**2)


      d = -rOne/dcmplx( pf2, 0.d0)
c
        fvi(1)=(fi(2)*((pf(1) - pf(4))*(vc(2) - cImag*vc(3)) + 
     -     (-pf(2) + cImag*pf(3))*(vc(1) - vc(4))) + 
     -  fi(1)*((-pf(2) + cImag*pf(3))*
     -      (vc(2) + cImag*vc(3)) + 
     -     (pf(1) - pf(4))*(vc(1) + vc(4))))*d


        fvi(2)=(fi(2)*((pf(2) + cImag*pf(3))*
     -      (-vc(2) + cImag*vc(3)) + 
     -     (pf(1) + pf(4))*(vc(1) - vc(4))) + 
     -  fi(1)*((pf(1) + pf(4))*(vc(2) + cImag*vc(3)) - 
     -     (pf(2) + cImag*pf(3))*(vc(1) + vc(4))))*d


        fvi(3)=(fi(3)*(-((pf(2) - cImag*pf(3))*
     -        (vc(2) + cImag*vc(3))) + 
     -     (pf(1) + pf(4))*(vc(1) - vc(4))) + 
     -  fi(4)*((pf(1) + pf(4))*
     -      (-vc(2) + cImag*vc(3)) + 
     -     (pf(2) - cImag*pf(3))*(vc(1) + vc(4))))*d


        fvi(4)=(fi(3)*(-((pf(1) - pf(4))*
     -        (vc(2) + cImag*vc(3))) + 
     -     (pf(2) + cImag*pf(3))*(vc(1) - vc(4))) + 
     -  fi(4)*((pf(2) + cImag*pf(3))*
     -      (-vc(2) + cImag*vc(3)) + 
     -     (pf(1) - pf(4))*(vc(1) + vc(4))))*d

      end
