      subroutine jioxxx(fi,fo, jio)
c
c This subroutine computes an off-shell vector current from an external
c fermion pair.  The vector boson propagator is given in Feynman gauge
c for a massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex fi(6)          : flow-in  fermion                   |fi>
c       complex fo(6)          : flow-out fermion                   <fo|
c
c output:
c       complex jio(6)         : vector current          j^mu(<fo|v|fi>)
c     
      implicit none
      double complex fi(6),fo(6),jio(6),d
      double precision q(0:3),q2


      double precision rZero, rOne
      parameter( rZero = 0.0d0, rOne = 1.0d0 )
      double complex cImag, cZero
      parameter( cImag = ( 0.0d0, 1.0d0 ), cZero = ( 0.0d0, 0.0d0 ) )


      jio(5) = fo(5)-fi(5)
      jio(6) = fo(6)-fi(6)

      q(0) = dble( jio(5))
      q(1) = dble( jio(6))
      q(2) = dimag(jio(6))
      q(3) = dimag(jio(5))
      q2 = q(0)**2-(q(1)**2+q(2)**2+q(3)**2)


         d = rOne/dcmplx( q2, 0.d0 )


        jio(1)=(fi(3)*fo(1) + fi(4)*fo(2) + 
     -  fi(1)*fo(3) + fi(2)*fo(4))*d


        jio(2)=(fi(4)*fo(1) + fi(3)*fo(2) - 
     -  fi(2)*fo(3) - fi(1)*fo(4))*d


        jio(3)=cImag*(-(fi(4)*fo(1)) + 
     -    fi(3)*fo(2) + fi(2)*fo(3) - 
     -    fi(1)*fo(4))*d


        jio(4)=(fi(3)*fo(1) - fi(4)*fo(2) - 
     -  fi(1)*fo(3) + fi(2)*fo(4))*d
c
      end
